"""

Contains the Client, which is the core object at the center of all foxford applications.

"""

from typing import Union, List

from .partials.partialuser import PartialUser
from .bases.baseuser import BaseUser
from .partials.partiallevel import PartialLevel
from .bases.baselevel import BaseLevel
from .users import User
from .level import Level
from .utilities.exceptions import BadRequest, NotFound, UserNotFound
from .utilities.requests import Requests

import asyncio

class Client:
    """
    Represents a Foxford client.

    Attributes:
        requests: The requests object, which is used to send requests to Foxford endpoints.
    """

    def __init__(self, cookie: str = None):
        """
        It is recommended to get cookies with EditThisCookie plugin thru export button

        Arguments:
            cookie: A Foxford cookie to login with. (_fox_session cookie)
        """
        self._requests: Requests = Requests()
        self.requests: Requests = self._requests

        self.loop = asyncio.get_event_loop()

        if cookie:
            self.set_cookie(cookie)
        else:
            raise Exception("No cookie provided")

    def __repr__(self):
        return f"<{self.__class__.__name__}>"

    # Authentication
    def set_cookie(self, cookie: str) -> None:
        """
        Authenticates the client with the passed Foxford cookie.
        This method does not send any requests and will not throw if the cookie is invalid.
        It is recommended to get cookies with EditThisCookie plugin thru export button

        Arguments:
            cookie: A Foxford cookie to login with.

        """
        self._requests.session.cookies["_fox_session"] = cookie

    # Users
    async def get_user(
            self, expand: bool = True
    ) -> Union[User, PartialUser]:
        """
        Grabs the authenticated user.

        Arguments:
            expand: Whether to return a User rather than a PartialUser

        Returns:
            The authenticated user.
        """
        authenticated_user_response = await self._requests.get(url="https://foxford.ru/api/user/me")
        authenticated_user_data = authenticated_user_response.json()

        if expand:
            return User(data=authenticated_user_data)
        else:
            return PartialUser(data=authenticated_user_data)
        
    async def get_level(
            self, expand: bool = True
            ) -> Union[Level, PartialLevel]:
        """
        Grabs the authenticated user's level.

        Arguments:
            expand: Whether to return a Level rather than a PartialLevel

        Returns:
            The authenticated user's level.
        """
        authenticated_user_response = await self._requests.get(url="https://foxford.ru/api/user/level")
        authenticated_user_data = authenticated_user_response.json()

        if expand:
            return Level(data=authenticated_user_data)
        else:
            return PartialLevel(data=authenticated_user_data)
        
    async def custom_request(self, method, url, *args):
        """
        Send method request to url with *args

        Example:
            await custom_request("GET", "https://foxford.ru/api/current_reminders")

        Arguments:
            method: one of the methods: GET, POST, PUT, DELETE
            url: url to send request to
        """
        authenticated_user_response = await self._requests.request(method=method, url=url, *args)
        authenticated_user_data = authenticated_user_response.json()

        return authenticated_user_data


"""

This module contains classes intended to parse and deal with data from Foxford client information endpoints.

"""

from .bases.baselevel import BaseLevel

class Level(BaseLevel):
    """
    Represents a single conversation.

    Attributes:
        gained_xp: The gained xp of user
        level: The level of user
        available_xp: The available xp of user
        progress: The level progress of user
    """

    def __init__(self, data: dict):
        """
        Arguments:
            data: The data from the request.
        """
        super().__init__(level=data["level"])

        self._data: dict = data

        self.level: int = data["level"]
        self.gained_xp: int = data["gained_xp"]
        self.available_xp: int = data["available_xp"]
        self.progress: int = data["progress"]

    def __repr__(self):
        return f"<{self.__class__.__name__} level={self.level!r}>"


from .baseitem import BaseItem

class BaseUser(BaseItem):
    """
    Represents a Foxford user ID.

    Attributes:
        id: The user ID.
    """

    def __init__(self, user_id: int):
        """
        Arguments:
            user_id: The user ID.
        """

        self.id: int = user_id


"""

foxford 
A modern, asynchronous wrapper for the Foxford's web API.

Copyright 2023-present frostx
License: MIT, see LICENSE

"""

# Find the original here: https://github.com/Rapptz/discord.py/blob/master/discord/__init__.py
# also copied from ro.py lmaooo

__title__ = "foxford"
__author__ = "frostx"
__license__ = "MIT"
__copyright__ = "Copyright 2023-present frostx"
__version__ = "1.1"

import logging
from typing import NamedTuple

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from .client import Client
from .utilities.exceptions import *

class VersionInfo(NamedTuple):
    """
    Represents the package's version info.
    """
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


version_info: VersionInfo = VersionInfo(
    major=2,
    minor=0,
    micro=0,
    releaselevel="release",
    serial=0
)

logging.getLogger(__name__).addHandler(logging.NullHandler())